@@ -2,6 +2,7 @@ from typing import (
     Callable,
     FrozenSet,
     List,
+    NoReturn,
     Set,
     Tuple,
     TypeVar,
@@ -12,7 +13,9 @@ _T = TypeVar("_T")
 _R = TypeVar("_R")
 
 
-def chain_sets(items: Tuple[FrozenSet[_T], ...]) -> Tuple[_T, ...]:
+def chain_items(
+    items: Tuple[Union[FrozenSet[_T], Tuple[_T, ...]], ...]
+) -> Tuple[_T, ...]:
     all_items: List[_T] = []
     for item_set in items:
         for item in item_set:
@@ -57,3 +60,9 @@ def to_tuple(item: Union[Tuple[str, ...], str]) -> Tuple[str, ...]:
     if isinstance(item, tuple):
         return item
     return (item,)
+
+
+def raise_or_value(item: Union[_T, Exception]) -> Union[_T, NoReturn]:
+    if isinstance(item, Exception):
+        raise item
+    return item
