@@ -2,6 +2,9 @@ from __future__ import (
     annotations,
 )
 
+from . import (
+    _utils,
+)
 from ._full_path import (
     FullPathModule,
 )
@@ -13,13 +16,44 @@ from grimp.application.ports.graph import (
     AbstractImportGraph,
 )
 from typing import (
-    cast,
     FrozenSet,
     NoReturn,
+    Optional,
     Tuple,
+    TypeVar,
     Union,
 )
 
+_T = TypeVar("_T")
+
+
+def _assert_bool(raw: _T) -> Union[bool, Exception]:
+    if isinstance(raw, bool):
+        return raw
+    return TypeError(
+        f"Expected `bool` but got `{type(raw)}`; This is a lib bug and should be reported."
+    )
+
+
+def _assert_str(raw: _T) -> Union[str, Exception]:
+    if isinstance(raw, str):
+        return raw
+    return TypeError(
+        f"Expected `str` but got `{type(raw)}`; This is a lib bug and should be reported."
+    )
+
+
+def _assert_opt_str_tuple(
+    raw: Optional[_T],
+) -> Union[Optional[Tuple[str, ...]], Exception]:
+    if raw is None:
+        return raw
+    if isinstance(raw, tuple):
+        return _utils.transform_items(raw, _assert_str)
+    return TypeError(
+        f"Expected `Optional[Tuple[str, ...]]` but got `{type(raw)}`; This is a lib bug and should be reported."
+    )
+
 
 @dataclass(frozen=True)  # type: ignore[misc]
 class _ImportGraph:  # type: ignore[no-any-unimported]
@@ -63,20 +97,22 @@ class ImportGraph:
         imported: FullPathModule,
         as_packages: bool,
     ) -> bool:
-        return cast(
-            bool,
+        result = _assert_bool(
             self._inner.graph.chain_exists(importer.module, imported.module, as_packages),  # type: ignore[misc]
         )
+        return _utils.raise_or_value(result)
 
     def find_shortest_chain(
         self,
         importer: FullPathModule,
         imported: FullPathModule,
-    ) -> Tuple[FullPathModule, ...]:
-        raw = cast(
-            Tuple[str],
+    ) -> Optional[Tuple[FullPathModule, ...]]:
+        _raw = _assert_opt_str_tuple(
             self._inner.graph.find_shortest_chain(importer.module, imported.module),  # type: ignore[misc]
         )
+        raw = _utils.raise_or_value(_raw)
+        if raw is None:
+            return None
         return tuple(FullPathModule.assert_module(r) for r in raw)
 
     def find_children(
